window.Umi = { UI: {}, Protocol: { SockChat: { Protocol: {} } } };

#include animate.js
#include common.js
#include context.js
#include emotes.js
#include messages.js
#include mszauth.js
#include server.js
#include txtrigs.js
#include utility.js
#include weeb.js
#include audio/autoplay.js
#include audio/context.js
#include controls/views.js
#include eeprom/eeprom.js
#include settings/backup.js
#include settings/settings.js
#include sound/context.js
#include sound/osukeys.js
#include sound/umisound.js
#include ui/chat-layout.js
#include ui/domaintrans.jsx
#include ui/hooks.js
#include ui/input-menus.js
#include ui/loading-overlay.jsx
#include ui/markup.js
#include ui/menus.js
#include ui/view.js
#include ui/settings.jsx
#include ui/toggles.js
#include ui/uploads.js

(async () => {
    const views = new MamiViewsControl({ body: document.body });
    const loadingOverlay = new Umi.UI.LoadingOverlay('spinner', 'Loading...');
    await views.push(loadingOverlay);

    loadingOverlay.setMessage('Loading environment...');
    try {
        window.futami = await FutamiCommon.load();
    } catch(ex) {
        console.error(ex);
        loadingOverlay.setIcon('cross');
        loadingOverlay.setHeader('Failed!');
        loadingOverlay.setMessage('Failed to load common settings!');
        return;
    }

    loadingOverlay.setMessage('Fetching credentials...');
    try {
        const auth = await MamiMisuzuAuth.update();
        if(!auth.ok)
            throw 'Authentication failed.';
    } catch(ex) {
        console.error(ex);
        location.assign(futami.get('login'));
        return;
    }

    setInterval(() => {
        MamiMisuzuAuth.update()
            .then(auth => {
                if(!auth.ok)
                    location.assign(futami.get('login'));
            })
    }, 600000);


    loadingOverlay.setMessage('Loading settings...');

    const settings = new MamiSettings('umi-');

    settings.define('style', 'string', 'dark');
    settings.define('compactView', 'boolean', false);
    settings.define('autoScroll', 'boolean', true);
    settings.define('closeTabConfirm', 'boolean', false);
    settings.define('showChannelList', 'boolean', false);
    settings.define('fancyInfo', 'boolean', true);
    settings.define('autoCloseUserContext', 'boolean', true);
    settings.define('enableParser', 'boolean', true);
    settings.define('enableEmoticons', 'boolean', true);
    settings.define('autoParseUrls', 'boolean', true);
    settings.define('preventOverflow', 'boolean', false);
    settings.define('expandTextBox', 'boolean', false);
    settings.define('eepromAutoInsert', 'boolean', true);
    settings.define('autoEmbedV1', 'boolean', false);
    settings.define('soundEnable', 'boolean', true, false, true);
    settings.define('soundPack', 'string', 'ajax-chat');
    settings.define('soundVolume', 'number', 80);
    settings.define('soundEnableJoin', 'boolean', true);
    settings.define('soundEnableLeave', 'boolean', true);
    settings.define('soundEnableError', 'boolean', true);
    settings.define('soundEnableServer', 'boolean', true);
    settings.define('soundEnableIncoming', 'boolean', true);
    settings.define('onlySoundOnMention', 'boolean', false);
    settings.define('soundEnableOutgoing', 'boolean', true);
    settings.define('soundEnablePrivate', 'boolean', true);
    settings.define('soundEnableForceLeave', 'boolean', true);
    settings.define('minecraft', ['no', 'yes', 'old'], 'no');
    settings.define('playSoundOnConnect', 'boolean', false);
    settings.define('windowsLiveMessenger', 'boolean', false);
    settings.define('seinfeld', 'boolean', false);
    settings.define('flashTitle', 'boolean', true);
    settings.define('showServerMsgInTitle', 'boolean', true);
    settings.define('playJokeSounds', 'boolean', true);
    settings.define('weeaboo', 'boolean', false);
    settings.define('motivationalImages', 'boolean', false);
    settings.define('motivationalVideos', 'boolean', false);
    settings.define('osuKeys', 'boolean', false);
    settings.define('osuKeysV2', ['no', 'yes', 'rng'], 'no');
    settings.define('explosionRadius', 'number', 20);
    settings.define('dumpPackets', 'boolean', FUTAMI_DEBUG);
    settings.define('neverUseWorker', 'boolean', false, false, true);
    settings.define('forceUseWorker', 'boolean', false, false, true);
    settings.define('marqueeAllNames', 'boolean', false);
    settings.define('tmpDisableOldThemeSys', 'boolean', false, false, true);
    settings.define('tmpSkipDomainPopUpThing', 'boolean', false, false, true);

    const noNotifSupport = !('Notification' in window);
    settings.define('enableNotifications', 'boolean', false, noNotifSupport, true);
    settings.define('notificationShowMessage', 'boolean', false, noNotifSupport);
    settings.define('notificationTriggers', 'string', '', noNotifSupport);


    loadingOverlay.setMessage('Loading sounds...');
    const soundCtx = new MamiSoundContext;

    try {
        const sounds = await futami.getJson('sounds2');
        if(Array.isArray(sounds.library))
            soundCtx.library.register(sounds.library, true);
        if(Array.isArray(sounds.packs))
            soundCtx.packs.register(sounds.packs, true);
    } catch(ex) {
        console.error(ex);
    }

    if(!await MamiDetectAutoPlay()) {
        settings.set('soundEnable', false);
        settings.virtualise('soundEnable');
    }

    settings.watch('soundEnable', (v, n, i) => {
        if(v) {
            if(!soundCtx.ready)
                soundCtx.reset();

            settings.touch('soundVolume');
            settings.touch('soundPack');

            soundCtx.packPlayer.playEvent('server');
        }

        soundCtx.muted = !v;
    });

    settings.watch('soundPack', (v, n, i) => {
        const packs = soundCtx.packs;
        if(!packs.has(v)) {
            settings.delete(n);
            return;
        }

        const player = soundCtx.packPlayer;
        player.loadPack(packs.get(v));
        if(!i) player.playEvent('server');
    });

    settings.watch('soundVolume', v => {
        soundCtx.volume = v / 100;
    })


    loadingOverlay.setMessage('Loading emoticons...');
    try {
        const emotes = await futami.getJson('emotes');
        MamiEmotes.loadLegacy(emotes);
    } catch(ex) {
        console.error(ex);
    }


    if(!settings.get('tmpSkipDomainPopUpThing'))
        await (() => {
            return new Promise((resolve) => {
                views.push(new MamiDomainTransition(soundCtx.library, () => {
                    (new MamiSettingsBackup(settings)).importUpload(document.body);
                }, () => {
                    settings.set('tmpSkipDomainPopUpThing', true);
                    views.pop();
                    resolve();
                }));
            });
        })();


    const onHashChange = () => {
        if(location.hash === '#reset') {
            settings.clear(true);
            location.assign('/');
        }
    };

    window.addEventListener('hashchange', onHashChange);
    onHashChange();

    window.addEventListener('keydown', ev => {
        if(ev.altKey && ev.shiftKey && (ev.key === 'R' || ev.key === 'r')) {
            Umi.Server.close();
            location.hash = 'reset';
        }
    });


    let eeprom;

    loadingOverlay.setMessage('Loading EEPROM...');
    try {
        await MamiEEPROM.init();
        eeprom = new EEPROM('1', futami.get('eeprom2'), MamiMisuzuAuth.getLine);
    } catch(ex) {
        console.error(ex);
        eeprom = undefined;
    }


    loadingOverlay.setMessage('Preparing UI...');

    const textTriggers = new MamiTextTriggers;

    // define this as late as possible'
    const ctx = new MamiContext({
        settings: settings,
        views: views,
        sound: soundCtx,
        textTriggers: textTriggers,
        eeprom: eeprom,
    });
    Object.defineProperty(window, 'mami', { enumerable: true, value: ctx });

    // should be dynamic when possible
    const layout = new Umi.UI.ChatLayout;
    await views.unshift(layout);

    Umi.UI.View.AccentReload();
    Umi.UI.Hooks.AddHooks();

    settings.watch('style', (v, n, i) => { if(!i) Umi.UI.View.AccentReload(); });
    settings.watch('compactView', (v, n, i) => { if(!i) Umi.UI.View.AccentReload(); });
    settings.watch('preventOverflow', v => document.body.classList.toggle('prevent-overflow', v));
    settings.watch('tmpDisableOldThemeSys', (v, n, i) => { if(!i) Umi.UI.View.AccentReload(); });

    settings.watch('minecraft', (v, n, i) => {
        if(v !== 'no') {
            if(i)
                soundCtx.library.play('minecraft:nether:enter');
            else
                Umi.Sound.Play('join');
        }
    });

    settings.watch('enableNotifications', v => {
        if(!v || !('Notification' in window)
            || (Notification.permission === 'granted' && Notification.permission !== 'denied'))
            return;

        Notification.requestPermission()
            .then(perm => {
                if(perm !== 'granted')
                    settings.set('enableNotifications', false);
            });
    });

    settings.watch('playJokeSounds', v => {
        if(!v) return;

        if(!textTriggers.hasTriggers())
            futami.getJson('texttriggers').then(trigInfos => textTriggers.addTriggers(trigInfos));
    });

    settings.watch('weeaboo', v => {
        if(v) Weeaboo.init();
    });

    settings.watch('osuKeysV2', (v, n, i) => {
        // migrate old value
        if(i) {
            if(settings.has('osuKeys')) {
                settings.set('osuKeysV2', settings.get('osuKeys') ? 'yes' : 'no');
                settings.delete('osuKeys');
                return;
            }
        }

        OsuKeys.setEnable(v !== 'no');
        OsuKeys.setRandomRate(v === 'rng');
    });


    loadingOverlay.setMessage('Building menus...');

    Umi.UI.Menus.Add('users', 'Users');
    Umi.UI.Menus.Add('channels', 'Channels', !settings.get('showChannelList'));
    Umi.UI.Menus.Add('settings', 'Settings');

    let sidebarAnimation = null;

    Umi.UI.Settings.Init();
    Umi.UI.Toggles.Add('menu-toggle', {
        'click': function() {
            const sidebar = $c('sidebar')[0],
                toggle = Umi.UI.Toggles.Get('menu-toggle'),
                toggleOpened = 'sidebar__selector-mode--menu-toggle-opened',
                toggleClosed = 'sidebar__selector-mode--menu-toggle-closed',
                isClosed = toggle.classList.contains(toggleClosed);

            if(sidebarAnimation !== null) {
                sidebarAnimation.cancel();
                sidebarAnimation = null;
            }

            if(isClosed) {
                toggle.classList.add(toggleOpened);
                toggle.classList.remove(toggleClosed);
            } else {
                toggle.classList.add(toggleClosed);
                toggle.classList.remove(toggleOpened);
            }

            let update;
            if(isClosed)
                update = function(t) {
                    sidebar.style.width = (40 + (220 * t)).toString() + 'px';
                };
            else
                update = function(t) {
                    sidebar.style.width = (260 - (220 * t)).toString() + 'px';
                };

            sidebarAnimation = MamiAnimate({
                duration: 500,
                easing: 'outExpo',
                update: update,
            });
        }
    }, 'Toggle Sidebar');

    Umi.UI.Toggles.Get('menu-toggle').classList.add('sidebar__selector-mode--menu-toggle-opened');

    Umi.UI.Toggles.Add('scroll', {
        'click': function() {
            settings.toggle('autoScroll');
        }
    }, 'Autoscroll');
    settings.watch('autoScroll', function(value) {
        Umi.UI.Toggles.Get('scroll').classList[value ? 'remove' : 'add']('sidebar__selector-mode--scroll-off');
    });

    if(window.innerWidth < 768)
        Umi.UI.Toggles.Get('menu-toggle').click();

    Umi.UI.Toggles.Add('audio', {
        'click': function() {
            settings.toggle('soundEnable');
        }
    }, 'Sounds');
    settings.watch('soundEnable', function(value) {
        Umi.UI.Toggles.Get('audio').classList[value ? 'remove' : 'add']('sidebar__selector-mode--audio-off');
    });

    Umi.UI.Toggles.Add('unembed', {
        'click': function() {
            const buttons = $qa('[data-embed="1"]');
            for(const button of buttons)
                button.click();
        }
    }, 'Unembed any embedded media');

    Umi.UI.Toggles.Add('clear', {
        'click': function() {
            if(confirm('ARE YOU SURE ABOUT THAT???')) {
                const limit = settings.get('explosionRadius');
                const explode = $e({
                    tag: 'img',
                    attrs: {
                        src: '//static.flash.moe/images/explode.gif',
                        alt: '',
                        style: {
                            position: 'absolute',
                            zIndex: 9001,
                            bottom: 0,
                            right: 0,
                            pointerEvents: 'none',
                        },
                        onLoad: function() {
                            setTimeout(function(){
                                $r(explode);
                            }, 1700);

                            soundCtx.library.play('misc:explode');
                        },
                    },
                });

                document.body.appendChild(explode);

                let backLog = Umi.Messages.All();
                backLog = backLog.slice(Math.max(backLog.length - limit, 0));

                Umi.Messages.Clear();

                for(const blMsg of backLog)
                    Umi.Messages.Add(blMsg);
            }
        }
    }, 'Clear Logs');


    if(eeprom !== undefined) {
        Umi.UI.Menus.Add('uploads', 'Upload History', !FUTAMI_DEBUG);

        const doUpload = async file => {
            const uploadEntry = Umi.UI.Uploads.create(file.name);
            const uploadTask = eeprom.create(file);

            uploadTask.onProgress(prog => uploadEntry.setProgress(prog.progress));
            uploadEntry.addOption('Cancel', () => uploadTask.abort());

            try {
                const fileInfo = await uploadTask.start();

                uploadEntry.hideOptions();
                uploadEntry.clearOptions();
                uploadEntry.removeProgress();

                uploadEntry.addOption('Open', fileInfo.url);
                uploadEntry.addOption('Insert', () => Umi.UI.Markup.InsertRaw(insertText, ''));
                uploadEntry.addOption('Delete', () => {
                    eeprom.delete(fileInfo)
                        .then(() => uploadEntry.remove())
                        .catch(ex => {
                            console.error(ex);
                            alert(ex);
                        });
                });

                let insertText;

                if(fileInfo.isImage()) {
                    insertText = `[img]${fileInfo.url}[/img]`;
                    uploadEntry.setThumbnail(fileInfo.thumb);
                } else if(fileInfo.isAudio()) {
                    insertText = `[audio]${fileInfo.url}[/audio]`;
                    uploadEntry.setThumbnail(fileInfo.thumb);
                } else if(fileInfo.isVideo()) {
                    insertText = `[video]${fileInfo.url}[/video]`;
                    uploadEntry.setThumbnail(fileInfo.thumb);
                } else
                    insertText = location.protocol + fileInfo.url;

                if(settings.get('eepromAutoInsert'))
                    Umi.UI.Markup.InsertRaw(insertText, '');
            } catch(ex) {
                if(!ex.aborted) {
                    console.error(ex);
                    alert(ex);
                }

                uploadEntry.remove();
            }
        };

        const uploadForm = $e({
            tag: 'input',
            attrs: {
                type: 'file',
                multiple: true,
                style: { display: 'none' },
                onchange: ev => {
                    for(const file of ev.target.files)
                        doUpload(file);
                },
            },
        });
        document.body.appendChild(uploadForm);

        Umi.UI.InputMenus.AddButton('upload', 'Upload', () => uploadForm.click());

        $i('umi-msg-text').onpaste = ev => {
            if(ev.clipboardData && ev.clipboardData.files.length > 0)
                for(const file of ev.clipboardData.files)
                    doUpload(file);
        };

        document.body.ondragenter = ev => {
            ev.preventDefault();
            ev.stopPropagation();
        };
        document.body.ondragover = ev => {
            ev.preventDefault();
            ev.stopPropagation();
        };
        document.body.ondragleave = ev => {
            ev.preventDefault();
            ev.stopPropagation();
        };
        document.body.ondrop = ev => {
            ev.preventDefault();
            ev.stopPropagation();

            if(ev.dataTransfer && ev.dataTransfer.files.length > 0)
                for(const file of ev.dataTransfer.files) {
                    if(file.name.slice(-5) === '.mami'
                        && confirm('This file appears to be a settings export. Do you want to import it? This will overwrite your existing settings!')) {
                        (new MamiSettingsBackup(settings)).importFile(file);
                        return;
                    }

                    doUpload(file);
                }
        };
    }

    Umi.UI.InputMenus.Add('markup', 'BB Code');
    Umi.UI.InputMenus.Add('emotes', 'Emoticons');

    window.addEventListener('beforeunload', function(ev) {
        if(settings.get('closeTabConfirm')) {
            ev.preventDefault();
            return ev.returnValue = 'Are you sure you want to close the tab?';
        }
    });


    loadingOverlay.setMessage('Connecting...');
    Umi.Server.open(views, settings);

    if(window.dispatchEvent)
        window.dispatchEvent(new Event('umi:connect'));
})();

#include compat.js
