#include channels.js
#include common.js
#include server.js
#include title.js
#include user.js
#include utility.js
#include sound/umisound.js
#include ui/channels.js
#include ui/messages.jsx
#include ui/users.js
#include ui/view.js

Umi.UI.Hooks = (function() {
    return {
        AddHooks: function() {
            const title = new MamiWindowTitle({
                getName: () => futami.get('title'),
            });

            window.addEventListener('focus', function() {
                title.clear();
            });


            Umi.Messages.OnRemove.push(function(msg) {
                Umi.UI.Messages.Remove(msg);
            });

            Umi.Messages.OnClear.push(function() {
                Umi.UI.Messages.RemoveAll();
            });

            Umi.Messages.OnAdd.push(function(msg) {
                Umi.UI.Channels.Unread(msg.getChannel());
                Umi.UI.Messages.Add(msg);

                if(!document.hidden)
                    return;

                if(mami.settings.get('flashTitle')) {
                    let titleText = ' ' + msg.getUser().getName(),
                        channel = Umi.Channels.Current() || null;
                    if(msg.getUser().isBot() && mami.settings.get('showServerMsgInTitle'))
                        titleText = ' ' + msg.getText();

                    if(channel !== null && channel.getName() !== msg.getChannel())
                        titleText += ' @ ' + channel.getName();

                    title.strobe([
                        `[ @] ${titleText}`,
                        `[@ ] ${titleText}`,
                    ]);
                }

                if(mami.settings.get('enableNotifications') && Umi.User.getCurrentUser() !== null) {
                    const triggers = (mami.settings.get('notificationTriggers') || '').toLowerCase().split(' '),
                        options = {};

                    triggers.push((Umi.User.getCurrentUser() || { getName: function() { return ''; } }).getName().toLowerCase());
                    options.body = 'Click here to see what they said.';

                    if(mami.settings.get('notificationShowMessage'))
                        options.body += "\n" + msg.getText();

                    const avatarUrl = futami.get('avatar');
                    if(avatarUrl.length > 0)
                        options.icon = avatarUrl.replace('{user:id}', msg.getUser().getId()).replace('{resolution}', '80').replace('{user:avatar_change}', msg.getUser().getAvatarTime().toString());

                    for(const trigger of triggers) {
                        const message = ' ' + msg.getText() + ' ';

                        if(trigger.trim() === '')
                            continue;

                        if(message.toLowerCase().indexOf(' ' + trigger + ' ') >= 0) {
                            new Notification('{0} mentioned you!'.replace('{0}', msg.getUser().getName()), options);
                            if(mami.settings.get('onlySoundOnMention'))
                                Umi.Sound.Play('incoming');
                            break;
                        }
                    }
                }
            });


            Umi.Users.OnAdd.push(function(user) {
                Umi.UI.Users.Add(user);
            });

            Umi.Users.OnRemove.push(function(user) {
                Umi.UI.Users.Remove(user);
            });

            Umi.Users.OnClear.push(function() {
                Umi.UI.Users.RemoveAll();
            });

            Umi.Users.OnUpdate.push(function(id, user) {
                Umi.UI.Users.Update(user);
            });


            Umi.Channels.OnAdd.push(function(channel) {
                Umi.UI.Channels.Add(channel);
            });

            Umi.Channels.OnRemove.push(function(channel) {
                Umi.UI.Channels.Remove(channel);
            });

            Umi.Channels.OnClear.push(function() {
                Umi.UI.Channels.RemoveAll();
            });

            Umi.Channels.OnUpdate.push(function(name, channel) {
                Umi.UI.Channels.Update(name, channel);
            });

            Umi.Channels.OnSwitch.push(function(name, channel) {
                Umi.UI.Channels.Reload(name === null);
            });


            window.addEventListener('keydown', function(ev) {
                if(!ev.target.matches('input, textarea'))
                    $i('umi-msg-text').focus();
            });

            $i('umi-msg-form').addEventListener('submit', ev => {
                ev.preventDefault();

                const textField = ev.target.elements.namedItem('text');
                if(textField instanceof HTMLTextAreaElement) {
                    let text = textField.value;
                    textField.value = '';

                    text = text.replace(/\t/g, '    ');
                    Umi.Server.sendMessage(text);
                }
            });

            $i('umi-msg-text').addEventListener('input', function(ev) {
                const elemInput = $i('umi-msg-text');
                const elemParent = elemInput.parentNode;
                let height = 40;

                if(mami.settings.get('expandTextBox') && elemInput.scrollHeight > elemInput.clientHeight)
                    height = elemInput.scrollHeight;

                if(height > 40)
                    elemParent.style.height = height.toString() + 'px';
                else
                    elemParent.style.height = null;
            });

            $i('umi-msg-text').addEventListener('keydown', function(ev) {
                if(ev.key === 'Tab' && (!ev.shiftKey || !ev.ctrlKey)) {
                    ev.preventDefault();

                    const text = Umi.UI.View.GetText();
                    if(text.length < 1)
                        return;

                    const start = Umi.UI.View.GetPosition();
                    let position = start,
                        snippet = '';
                    while(position >= 0 && text.charAt(position - 1) !== ' ' && text.charAt(position - 1) !== "\n") {
                        --position;
                        snippet = text.charAt(position) + snippet;
                    }

                    let insertText = undefined;

                    if(snippet.indexOf(':') === 0) {
                        let emoteRank = 0;
                        if(Umi.User.hasCurrentUser())
                            emoteRank = Umi.User.getCurrentUser().getRank();
                        const emotes = MamiEmotes.findByName(emoteRank, snippet.substring(1), true);
                        if(emotes.length > 0)
                            insertText = ':' + emotes[0] + ':';
                    } else {
                        const users = Umi.Users.Find(snippet);
                        if(users.length === 1)
                            insertText = users[0].getName();
                    }

                    if(insertText !== undefined) {
                        Umi.UI.View.SetText(text.slice(0, start - snippet.length) + text.slice(start));
                        Umi.UI.View.SetPosition(start - snippet.length);
                        Umi.UI.View.EnterAtCursor(insertText);
                        Umi.UI.View.SetPosition(Umi.UI.View.GetPosition() + insertText.length);
                        Umi.UI.View.SetPosition(Umi.UI.View.GetPosition(), true);
                    }

                    return;
                }

                if((ev.key === 'Enter' || ev.key === 'NumpadEnter') && !ev.shiftKey) {
                    ev.preventDefault();
                    $i('umi-msg-form').requestSubmit();
                    return;
                }
            });
        },
    };
})();
